#!/usr/bin/env node
"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const ora = require("ora");
const mri = require("mri");
const glob = require("glob");
const _1 = require(".");
const nullable_1 = require("@softwareventures/nullable");
const unknown_1 = require("unknown");
const array_1 = require("@softwareventures/array");
const LIBRARY_NAME = "precise-commits";
const config = mri(process.argv.slice(2));
/**
 * If the user provided one or more glob patterns to match against, ensure that there are
 * applicable files available
 */
let filesWhitelist = null;
if ((0, unknown_1.hasProperty)(config, "whitelist")) {
    if ((0, array_1.isArray)(config.whitelist)) {
        filesWhitelist = (0, array_1.concatMap)(config.whitelist, entry => glob.sync(String(entry)));
    }
    else {
        filesWhitelist = glob.sync(String(config.whitelist));
    }
    if (!filesWhitelist || !filesWhitelist.length) {
        console.error(`Error: No files match the glob pattern(s) you provided for --whitelist -> "${config.whitelist}"`);
        process.exit(1);
    }
    filesWhitelist = filesWhitelist.map(path_1.normalize);
}
/**
 * If the user specifies at least one SHA, perform some validation and
 * apply some defaults
 */
let base = null;
let head = null;
if ((0, unknown_1.hasProperty)(config, "base") || (0, unknown_1.hasProperty)(config, "head")) {
    if (!(0, unknown_1.hasProperty)(config, "base") || config.base == null) {
        console.error(`Error: When giving a value of --head, you must also give a value for --base`);
        process.exit(1);
    }
    if ((0, unknown_1.hasProperty)(config, "head") && config.head != null) {
        head = String(config.head);
    }
    else {
        /**
         * If the user only specified `--base`, set the value of `--head` to be "HEAD"
         */
        head = "HEAD";
    }
    base = String(config.base);
}
const checkOnly = (0, unknown_1.hasProperty)(config, "check-only") ? Boolean(config["check-only"]) : false;
const formatter = (0, unknown_1.hasProperty)(config, "formatter")
    ? String((_a = config.formatter) !== null && _a !== void 0 ? _a : "prettier")
    : "prettier";
const options = {
    checkOnly,
    filesWhitelist,
    base,
    head,
    formatter
};
const primarySpinner = ora(` Running ${LIBRARY_NAME}...`);
const modifiedFilesSpinner = ora(" Detecting modified files from git...");
const spinnersByFilename = new Map();
let shouldErrorOut = false;
(0, _1.main)(process.cwd(), options, {
    onInit(workingDirectory) {
        primarySpinner.start();
        modifiedFilesSpinner.start();
    },
    onModifiedFilesDetected(modifiedFilenames) {
        if (!modifiedFilenames || !modifiedFilenames.length) {
            return;
        }
        modifiedFilesSpinner.succeed(` ${LIBRARY_NAME}: ${modifiedFilenames.length} modified file(s) found`);
    },
    onBegunProcessingFile(filename, index, totalFiles) {
        spinnersByFilename.set(filename, ora()
            .start()
            .succeed(` [${index + 1}/${totalFiles}] Processing file: ${filename}`));
    },
    onFinishedProcessingFile(filename, index, status) {
        const spinner = spinnersByFilename.get(filename);
        switch (status) {
            case "UPDATED":
                (0, nullable_1.notNull)(spinner).succeed(`       --> Updated formatting in: ${filename}`);
                break;
            case "NOT_UPDATED":
                (0, nullable_1.notNull)(spinner).info(`       --> No formatting changes required in: ${filename}`);
                break;
            case "INVALID_FORMATTING":
                /**
                 * If --check-only is passed as a CLI argument, the script will error out.
                 */
                if (options.checkOnly) {
                    shouldErrorOut = true;
                }
                (0, nullable_1.notNull)(spinner).fail(`       --> Invalid formatting detected in: ${filename}`);
                break;
        }
    },
    onError(err) {
        modifiedFilesSpinner.fail(` ${LIBRARY_NAME}: An Error occurred\n`);
        console.error(err);
        console.log("\n");
        primarySpinner.stop();
        return process.exit(1);
    },
    onComplete(totalFiles) {
        if (!totalFiles) {
            modifiedFilesSpinner.info(` ${LIBRARY_NAME}: No matching modified files detected.
        
  --> If you feel that one or more files should be showing up here, be sure to first check what file extensions prettier supports, and whether or not you have included those files in a .prettierignore file

        `);
            primarySpinner.stop();
            return process.exit(shouldErrorOut ? 1 : 0);
        }
        if (options.checkOnly) {
            primarySpinner.succeed(" Checks complete 🎉");
        }
        else {
            primarySpinner.succeed(" Formatting complete 🎉");
        }
        return process.exit(shouldErrorOut ? 1 : 0);
    }
});
//# sourceMappingURL=cli.js.map