"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getModifiedFilenames = exports.getDiffForFile = exports.workingTree = exports.index = exports.resolveNearestGitDirectoryParent = void 0;
const find_up_1 = require("find-up");
const path_1 = require("path");
const utils_1 = require("./utils");
const unknown_1 = require("unknown");
function resolveNearestGitDirectoryParent(workingDirectory) {
    const gitDirectoryPath = (0, find_up_1.sync)(".git", { cwd: workingDirectory, type: "directory" });
    if (!gitDirectoryPath) {
        throw new Error("No .git directory found");
    }
    return (0, path_1.dirname)(gitDirectoryPath);
}
exports.resolveNearestGitDirectoryParent = resolveNearestGitDirectoryParent;
exports.index = Symbol("index");
exports.workingTree = Symbol("working-tree");
function getDiffForFile(gitDirectoryParent, fullPath, base, head) {
    if (head === exports.index) {
        if (base == null) {
            return (0, utils_1.runCommandSync)("git", ["diff", "--unified=0", "--cached", "--", fullPath], gitDirectoryParent).stdout;
        }
        else {
            return (0, utils_1.runCommandSync)("git", ["diff", "--unified=0", "--cached", base, "--", fullPath], gitDirectoryParent).stdout;
        }
    }
    else if (head === exports.workingTree) {
        if (base == null) {
            try {
                return (0, utils_1.runCommandSync)("git", ["diff", "--unified=0", "HEAD", "--", fullPath], gitDirectoryParent).stdout;
            }
            catch (err) {
                //If there has never been a commit before, there will be no HEAD to compare
                // to. Use the special empty tree hash value instead:
                // https://stackoverflow.com/questions/9765453/is-gits-semi-secret-empty-tree-object-reliable-and-why-is-there-not-a-symbolic
                if ((0, unknown_1.hasProperty)(err, "message") &&
                    typeof err.message === "string" &&
                    err.message.includes("fatal: bad revision")) {
                    return (0, utils_1.runCommandSync)("git", ["diff", "--unified=0", SPECIAL_EMPTY_TREE_COMMIT_HASH, "--", fullPath], gitDirectoryParent).stdout;
                }
                else {
                    throw err;
                }
            }
        }
        else {
            return (0, utils_1.runCommandSync)("git", ["diff", "--unified=0", base, "--", fullPath], gitDirectoryParent).stdout;
        }
    }
    else if (base == null) {
        throw new Error("Invalid argument");
    }
    else {
        return (0, utils_1.runCommandSync)("git", ["diff", "--unified=0", base, head, fullPath], gitDirectoryParent).stdout;
    }
}
exports.getDiffForFile = getDiffForFile;
/**
 * Output of `git diff-index --help`:
 *
 * --diff-filter=[(A|C|D|M|R|T|U|X|B)...[*]]
      Select only files that are Added (A), Copied (C), Deleted (D), Modified (M), Renamed (R), have
      their type (i.e. regular file, symlink, submodule, ...) changed (T), are Unmerged (U), are
      Unknown (X), or have had their pairing Broken (B). Any combination of the filter characters
      (including none) can be used. When * (All-or-none) is added to the combination, all paths are
      selected if there is any file that matches other criteria in the comparison; if there is no
      file that matches other criteria, nothing is selected.

      Also, these upper-case letters can be downcased to exclude. E.g.  --diff-filter=ad excludes
      added and deleted paths.
 *
 */
/**
 * const DIFF_INDEX_FILTER = 'ACDMRTUXB';
 * NOTE: We are only explicitly testing "Modified" and "Added" files for now...
 */
const DIFF_INDEX_FILTER = "AM";
const SPECIAL_EMPTY_TREE_COMMIT_HASH = "4b825dc642cb6eb9a060e54bf8d69288fbee4904";
function getModifiedFilenames(gitDirectoryParent, base, head) {
    let diffIndexOutput;
    if (base && head) {
        /**
         * We are grabbing the files between the two given commit SHAs
         */
        diffIndexOutput = (0, utils_1.runCommandSync)("git", ["diff", "--name-status", `--diff-filter=${DIFF_INDEX_FILTER}`, base, head], gitDirectoryParent).stdout;
    }
    else {
        /**
         * No commit SHAs given, we assume we are attempting to evaluate staged files,
         * and so need to determine if there is HEAD SHA available.
         */
        let head = "";
        try {
            head = (0, utils_1.runCommandSync)("git", ["rev-parse", "--verify", "HEAD"], gitDirectoryParent).stdout.replace("\n", "");
        }
        catch (err) {
            /**
             * If there has never been a commit before, there will be no HEAD to compare
             * to. Use the special empty tree hash value instead:
             * https://stackoverflow.com/questions/9765453/is-gits-semi-secret-empty-tree-object-reliable-and-why-is-there-not-a-symbolic
             */
            if ((0, unknown_1.hasProperty)(err, "message") &&
                typeof err.message === "string" &&
                err.message.includes("fatal: Needed a single revision")) {
                head = SPECIAL_EMPTY_TREE_COMMIT_HASH;
            }
            else {
                throw err;
            }
        }
        diffIndexOutput = (0, utils_1.runCommandSync)("git", ["diff-index", "--cached", "--name-status", `--diff-filter=${DIFF_INDEX_FILTER}`, head], gitDirectoryParent).stdout;
    }
    const allFiles = parseDiffIndexOutput(diffIndexOutput);
    return allFiles.map(r => r.filename);
}
exports.getModifiedFilenames = getModifiedFilenames;
function parseDiffIndexOutput(stdout) {
    const lines = stdout.split("\n");
    return lines.filter(Boolean).map(line => {
        const parts = line.split("\t");
        return {
            filename: parts[1],
            diffFilterChar: parts[0]
        };
    });
}
//# sourceMappingURL=git-utils.js.map