"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const path_1 = require("path");
const git_utils_1 = require("./git-utils");
const utils_1 = require("./utils");
const modified_file_1 = require("./modified-file");
const prettier_1 = require("./precise-formatters/prettier");
const unknown_1 = require("./unknown");
/**
 * LIBRARY
 */
function main(workingDirectory, additionalOptions, callbacks = {
    onInit() { },
    onModifiedFilesDetected() { },
    onBegunProcessingFile() { },
    onFinishedProcessingFile() { },
    onError() { },
    onComplete() { }
}) {
    try {
        /**
         * Merge user-given and default options.
         */
        const options = Object.assign({
            filesWhitelist: null,
            base: null,
            head: null,
            checkOnly: false,
            formatter: "prettier"
        }, additionalOptions);
        /**
         * Note: Will be exposed as an option if/when new formatters are added.
         */
        if (options.formatter !== "prettier") {
            throw new Error(`The only supported value for "formatter" option is "prettier"`);
        }
        const selectedFormatter = prettier_1.preciseFormatterPrettier;
        callbacks.onInit(workingDirectory);
        /**
         * Resolve the relevant .git directory's parent directory up front, as we will need this when
         * executing various `git` commands.
         */
        const gitDirectoryParent = (0, git_utils_1.resolveNearestGitDirectoryParent)(workingDirectory);
        /**
         * We fundamentally check whether or not the file extensions are supported by the given formatter,
         * whether or not they are included in the optional `filesWhitelist` array, and that the user
         * has not chosen to ignore them via any supported "ignore" mechanism of the formatter.
         */
        const modifiedFilenames = (0, git_utils_1.getModifiedFilenames)(gitDirectoryParent, options.base, options.head)
            .map(path => (0, path_1.join)(gitDirectoryParent, path))
            .map(path => (0, path_1.relative)(workingDirectory, path))
            .filter(path => !(0, path_1.isAbsolute)(path))
            .filter(selectedFormatter.hasSupportedFileExtension)
            .filter((0, utils_1.generateFilesWhitelistPredicate)(options.filesWhitelist))
            .filter(selectedFormatter.generateIgnoreFilePredicate(workingDirectory));
        /**
         * Report on the the total number of relevant files.
         */
        const totalFiles = modifiedFilenames.length;
        callbacks.onModifiedFilesDetected(modifiedFilenames);
        /**
         * Process each file synchronously.
         */
        modifiedFilenames.forEach((filename, fileIndex) => {
            callbacks.onBegunProcessingFile(filename, fileIndex, totalFiles);
            const heads = options.head == null ? [git_utils_1.index, git_utils_1.workingTree] : [options.head];
            for (const head of heads) {
                /**
                 * Read the modified file contents and resolve the relevant formatter.
                 */
                const modifiedFile = new modified_file_1.ModifiedFile({
                    fullPath: (0, path_1.join)(workingDirectory, filename),
                    gitDirectoryParent,
                    base: options.base,
                    head,
                    selectedFormatter
                });
                /**
                 * To avoid unnecessary issues with 100% valid files producing issues when parts
                 * of them are reformatted in isolation, we first check the whole file to see if
                 * it is already formatted. This could also allow us to skip unnecessary git diff
                 * analysis work.
                 */
                if (modifiedFile.isAlreadyFormatted()) {
                    return callbacks.onFinishedProcessingFile(filename, fileIndex, "NOT_UPDATED");
                }
                /**
                 * Calculate what character ranges have been affected in the modified file.
                 * If any of the analysis threw an error for any reason, it will be returned
                 * from the method so we can handle it here.
                 */
                const { err } = modifiedFile.calculateModifiedCharacterRanges();
                if (err) {
                    if (err.message === utils_1.NO_LINE_CHANGE_DATA_ERROR) {
                        return callbacks.onFinishedProcessingFile(filename, fileIndex, "NOT_UPDATED");
                    }
                    /**
                     * Unexpected error, bubble up to the main onError handler
                     */
                    throw err;
                }
                /**
                 * "CHECK ONLY MODE"
                 */
                if (options.checkOnly) {
                    if (!modifiedFile.hasValidFormattingForCharacterRanges()) {
                        return callbacks.onFinishedProcessingFile(filename, fileIndex, "INVALID_FORMATTING");
                    }
                    else {
                        return callbacks.onFinishedProcessingFile(filename, fileIndex, "NOT_UPDATED");
                    }
                }
                /**
                 * "FORMAT MODE"
                 */
                modifiedFile.formatCharacterRangesWithinContents();
                if (!modifiedFile.shouldContentsBeUpdatedOnDisk()) {
                    return callbacks.onFinishedProcessingFile(filename, fileIndex, "NOT_UPDATED");
                }
                /**
                 * Write the file back to disk and report.
                 */
                modifiedFile.updateFileOnDisk();
            }
            return callbacks.onFinishedProcessingFile(filename, fileIndex, "UPDATED");
        });
        /**
         * Report that all files have finished processing.
         */
        callbacks.onComplete(totalFiles);
    }
    catch (err) {
        /**
         * Report any unhandled errors.
         */
        callbacks.onError((0, unknown_1.assertInstanceOf)(err, Error));
    }
}
exports.main = main;
//# sourceMappingURL=index.js.map