"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateFilesWhitelistPredicate = exports.runCommandSync = exports.calculateCharacterRangesFromLineChanges = exports.extractLineChangeData = exports.NO_LINE_CHANGE_DATA_ERROR = void 0;
const execa = require("execa");
exports.NO_LINE_CHANGE_DATA_ERROR = "No line change data could be detected";
/**
 * Addition `start` number included in the range,
 * removal `start` is the line before
 */
function extractLineChangeData(diffData) {
    const lineChanges = diffData.match(/@@.*@@/g);
    if (!lineChanges) {
        throw new Error(exports.NO_LINE_CHANGE_DATA_ERROR);
    }
    const lineChangeData = {
        removals: [],
        additions: []
    };
    lineChanges.forEach(lineChange => {
        const d = lineChange.match(/(@@ )-(\d+,?\d*)( )\+(\d+,?\d*)( @@)/);
        if (!d) {
            throw new Error("The detected line change data could be not be parsed");
        }
        const [removalStartLine, noOfLinesRemoved = 1] = d[2].split(",").map(s => parseInt(s, 10));
        const [additionStartLine, noOfLinesAdded = 1] = d[4].split(",").map(s => parseInt(s, 10));
        if (noOfLinesRemoved > 0) {
            lineChangeData.removals.push({
                start: removalStartLine,
                noOfLines: noOfLinesRemoved
            });
        }
        if (noOfLinesAdded > 0) {
            lineChangeData.additions.push({
                start: additionStartLine,
                noOfLines: noOfLinesAdded
            });
        }
    });
    return lineChangeData;
}
exports.extractLineChangeData = extractLineChangeData;
function findCharacterIndexOfLine(fileContents, startCharIndex, lineCount) {
    let charIndex = startCharIndex;
    let lineIndex = 0;
    while (lineIndex < lineCount && charIndex < fileContents.length) {
        const char = fileContents[charIndex];
        const nextChar = fileContents[charIndex + 1];
        if (char === "\n" || char === "\r") {
            ++lineIndex;
            if (char === "\r" && nextChar === "\n") {
                ++charIndex; // skip next character
            }
        }
        ++charIndex;
    }
    return charIndex;
}
function calculateCharacterRangesFromLineChanges(lineChangeData, fileContents) {
    let charIndex = 0;
    let lineIndex = 0;
    return lineChangeData.additions.map(added => {
        const rangeStart = findCharacterIndexOfLine(fileContents, charIndex, added.start - lineIndex - 1);
        const rangeEnd = findCharacterIndexOfLine(fileContents, rangeStart, added.noOfLines);
        charIndex = rangeEnd;
        lineIndex = added.start + added.noOfLines - 1;
        return { rangeStart, rangeEnd };
    });
}
exports.calculateCharacterRangesFromLineChanges = calculateCharacterRangesFromLineChanges;
function runCommandSync(command, args, workingDirectory = process.cwd()) {
    return execa.sync(command, args, { cwd: workingDirectory });
}
exports.runCommandSync = runCommandSync;
function generateFilesWhitelistPredicate(filesWhitelist) {
    if (!filesWhitelist) {
        return () => true;
    }
    return file => filesWhitelist.includes(file);
}
exports.generateFilesWhitelistPredicate = generateFilesWhitelistPredicate;
//# sourceMappingURL=utils.js.map